@@ -1,12 +1,15 @@
 import collections.abc
 import sys
+import types
 from contextlib import nullcontext
+from datetime import timedelta
 from functools import partial
 from io import BytesIO, StringIO
 from pathlib import Path
 from typing import (
     IO,
     AbstractSet,
+    Annotated,
     Any,
     AnyStr,
     BinaryIO,
@@ -15,13 +18,18 @@ from typing import (
     ContextManager,
     Dict,
     ForwardRef,
+    FrozenSet,
+    Iterable,
     Iterator,
     List,
+    Literal,
     Mapping,
     MutableMapping,
     Optional,
+    Protocol,
     Sequence,
     Set,
+    Sized,
     TextIO,
     Tuple,
     Type,
@@ -30,16 +38,19 @@ from typing import (
 )
 
 import pytest
-from pytest import MonkeyPatch
+from typing_extensions import LiteralString
 
 from typeguard import (
     CollectionCheckStrategy,
     ForwardRefPolicy,
     TypeCheckError,
+    TypeCheckMemo,
     TypeHintWarning,
     check_type,
-    config,
+    check_type_internal,
+    suppress_type_checks,
 )
+from typeguard._checkers import is_typeddict
 from typeguard._utils import qualified_name
 
 from . import (
@@ -47,39 +58,30 @@ from . import (
     Employee,
     JSONType,
     Parent,
-    RuntimeProtocol,
-    StaticProtocol,
     TChild,
     TIntStr,
     TParent,
     TTypingConstrained,
     myint,
+    mylist,
 )
 
 if sys.version_info >= (3, 11):
-    from typing import LiteralString
-
     SubclassableAny = Any
 else:
     from typing_extensions import Any as SubclassableAny
-    from typing_extensions import LiteralString
 
 if sys.version_info >= (3, 10):
-    from typing import TypeGuard
+    from typing import Concatenate, ParamSpec, TypeGuard
 else:
-    from typing_extensions import TypeGuard
+    from typing_extensions import Concatenate, ParamSpec, TypeGuard
 
-if sys.version_info >= (3, 9):
-    from typing import Annotated
-else:
-    from typing_extensions import Annotated
-
-if sys.version_info >= (3, 8):
-    from typing import Literal, TypedDict
-else:
-    from typing_extensions import Literal, TypedDict
+P = ParamSpec("P")
 
 
+@pytest.mark.skipif(
+    sys.version_info >= (3, 13), reason="AnyStr is deprecated on Python 3.13"
+)
 class TestAnyStr:
     @pytest.mark.parametrize(
         "value", [pytest.param("bar", id="str"), pytest.param(b"bar", id="bytes")]
@@ -178,8 +180,8 @@ class TestCallable:
         pytest.raises(
             TypeCheckError, check_type, some_callable, Callable[[int, str], int]
         ).match(
-            r"has too many arguments in its declaration; expected 2 but 3 "
-            r"argument\(s\) declared"
+            r"has too many mandatory positional arguments in its declaration; expected "
+            r"2 but 3 mandatory positional argument\(s\) declared"
         )
 
     def test_mandatory_kwonlyargs(self):
@@ -258,6 +260,16 @@ class TestCallable:
         """
         check_type([].append, Callable[[int], Any])
 
+    def test_concatenate(self):
+        """Test that ``Concatenate`` in the arglist is ignored."""
+        check_type([].append, Callable[Concatenate[object, P], Any])
+
+    def test_positional_only_arg_with_default(self):
+        def some_callable(x: int = 1, /) -> None:
+            pass
+
+        check_type(some_callable, Callable[[int], Any])
+
 
 class TestLiteral:
     def test_literal_union(self):
@@ -319,26 +331,22 @@ class TestMapping:
             f"instance of str"
         )
 
-    def test_bad_key_type_full_check(self, monkeypatch: MonkeyPatch):
-        monkeypatch.setattr(
-            config, "collection_check_strategy", CollectionCheckStrategy.ALL_ITEMS
-        )
+    def test_bad_key_type_full_check(self):
         pytest.raises(
             TypeCheckError,
             check_type,
             {"x": 1, 3: 2},
             Mapping[str, int],
+            collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS,
         ).match("key 3 of dict is not an instance of str")
 
-    def test_bad_value_type_full_check(self, monkeypatch: MonkeyPatch):
-        monkeypatch.setattr(
-            config, "collection_check_strategy", CollectionCheckStrategy.ALL_ITEMS
-        )
+    def test_bad_value_type_full_check(self):
         pytest.raises(
             TypeCheckError,
             check_type,
             {"x": 1, "y": "a"},
             Mapping[str, int],
+            collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS,
         ).match("value of key 'y' of dict is not an instance of int")
 
     def test_any_value_type(self):
@@ -408,25 +416,32 @@ class TestDict:
             "value of key 'x' of dict is not an instance of int"
         )
 
-    def test_bad_key_type_full_check(self, monkeypatch: MonkeyPatch):
-        monkeypatch.setattr(
-            config, "collection_check_strategy", CollectionCheckStrategy.ALL_ITEMS
-        )
-        pytest.raises(TypeCheckError, check_type, {"x": 1, 3: 2}, Dict[str, int]).match(
-            "key 3 of dict is not an instance of str"
-        )
+    def test_bad_key_type_full_check(self):
+        pytest.raises(
+            TypeCheckError,
+            check_type,
+            {"x": 1, 3: 2},
+            Dict[str, int],
+            collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS,
+        ).match("key 3 of dict is not an instance of str")
 
-    def test_bad_value_type_full_check(self, monkeypatch: MonkeyPatch):
-        monkeypatch.setattr(
-            config, "collection_check_strategy", CollectionCheckStrategy.ALL_ITEMS
-        )
+    def test_bad_value_type_full_check(self):
         pytest.raises(
             TypeCheckError,
             check_type,
             {"x": 1, "y": "a"},
             Dict[str, int],
+            collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS,
         ).match("value of key 'y' of dict is not an instance of int")
 
+    def test_custom_dict_generator_items(self):
+        class CustomDict(dict):
+            def items(self):
+                for key in self:
+                    yield key, self[key]
+
+        check_type(CustomDict(a=1), Dict[str, int])
+
 
 class TestTypedDict:
     @pytest.mark.parametrize(
@@ -458,10 +473,18 @@ class TestTypedDict:
                 r'dict has unexpected extra key\(s\): "foo"',
                 id="unknown_key",
             ),
+            pytest.param(
+                None,
+                True,
+                "is not a dict",
+                id="not_dict",
+            ),
         ],
     )
-    def test_typed_dict(self, value, total: bool, error_re: Optional[str]):
-        class DummyDict(TypedDict, total=total):
+    def test_typed_dict(
+        self, value, total: bool, error_re: Optional[str], typing_provider
+    ):
+        class DummyDict(typing_provider.TypedDict, total=total):
             x: int
             y: str
 
@@ -470,14 +493,56 @@ class TestTypedDict:
         else:
             check_type(value, DummyDict)
 
-    def test_inconsistent_keys_invalid(self):
-        class DummyDict(TypedDict):
+    def test_inconsistent_keys_invalid(self, typing_provider):
+        class DummyDict(typing_provider.TypedDict):
             x: int
 
         pytest.raises(
             TypeCheckError, check_type, {"x": 1, "y": 2, b"z": 3}, DummyDict
         ).match(r'dict has unexpected extra key\(s\): "y", "b\'z\'"')
 
+    def test_notrequired_pass(self, typing_provider):
+        try:
+            NotRequired = typing_provider.NotRequired
+        except AttributeError:
+            pytest.skip(f"'NotRequired' not found in {typing_provider.__name__!r}")
+
+        class DummyDict(typing_provider.TypedDict):
+            x: int
+            y: NotRequired[int]
+            z: "NotRequired[int]"
+
+        check_type({"x": 8}, DummyDict)
+
+    def test_notrequired_fail(self, typing_provider):
+        try:
+            NotRequired = typing_provider.NotRequired
+        except AttributeError:
+            pytest.skip(f"'NotRequired' not found in {typing_provider.__name__!r}")
+
+        class DummyDict(typing_provider.TypedDict):
+            x: int
+            y: NotRequired[int]
+            z: "NotRequired[int]"
+
+        with pytest.raises(
+            TypeCheckError, match=r"value of key 'y' of dict is not an instance of int"
+        ):
+            check_type({"x": 1, "y": "foo"}, DummyDict)
+
+        with pytest.raises(
+            TypeCheckError, match=r"value of key 'z' of dict is not an instance of int"
+        ):
+            check_type({"x": 1, "y": 6, "z": "foo"}, DummyDict)
+
+    def test_is_typeddict(self, typing_provider):
+        # Ensure both typing.TypedDict and typing_extensions.TypedDict are recognized
+        class DummyDict(typing_provider.TypedDict):
+            x: int
+
+        assert is_typeddict(DummyDict)
+        assert not is_typeddict(dict)
+
 
 class TestList:
     def test_bad_type(self):
@@ -496,13 +561,14 @@ class TestList:
             "list is not an instance of int"
         )
 
-    def test_full_check_fail(self, monkeypatch: MonkeyPatch):
-        monkeypatch.setattr(
-            config, "collection_check_strategy", CollectionCheckStrategy.ALL_ITEMS
-        )
-        pytest.raises(TypeCheckError, check_type, [1, 2, "bb"], List[int]).match(
-            "list is not an instance of int"
-        )
+    def test_full_check_fail(self):
+        pytest.raises(
+            TypeCheckError,
+            check_type,
+            [1, 2, "bb"],
+            List[int],
+            collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS,
+        ).match("list is not an instance of int")
 
 
 class TestSequence:
@@ -526,13 +592,14 @@ class TestSequence:
             "list is not an instance of int"
         )
 
-    def test_full_check_fail(self, monkeypatch: MonkeyPatch):
-        monkeypatch.setattr(
-            config, "collection_check_strategy", CollectionCheckStrategy.ALL_ITEMS
-        )
-        pytest.raises(TypeCheckError, check_type, [1, 2, "bb"], Sequence[int]).match(
-            "list is not an instance of int"
-        )
+    def test_full_check_fail(self):
+        pytest.raises(
+            TypeCheckError,
+            check_type,
+            [1, 2, "bb"],
+            Sequence[int],
+            collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS,
+        ).match("list is not an instance of int")
 
 
 class TestAbstractSet:
@@ -560,13 +627,14 @@ class TestAbstractSet:
             "set is not an instance of int"
         )
 
-    def test_full_check_fail(self, monkeypatch: MonkeyPatch):
-        monkeypatch.setattr(
-            config, "collection_check_strategy", CollectionCheckStrategy.ALL_ITEMS
-        )
-        pytest.raises(TypeCheckError, check_type, {1, 2, "bb"}, AbstractSet[int]).match(
-            "set is not an instance of int"
-        )
+    def test_full_check_fail(self):
+        pytest.raises(
+            TypeCheckError,
+            check_type,
+            {1, 2, "bb"},
+            AbstractSet[int],
+            collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS,
+        ).match("set is not an instance of int")
 
 
 class TestSet:
@@ -584,12 +652,45 @@ class TestSet:
             "set is not an instance of int"
         )
 
-    def test_full_check_fail(self, monkeypatch: MonkeyPatch):
-        monkeypatch.setattr(
-            config, "collection_check_strategy", CollectionCheckStrategy.ALL_ITEMS
+    def test_full_check_fail(self):
+        pytest.raises(
+            TypeCheckError,
+            check_type,
+            {1, 2, "bb"},
+            Set[int],
+            collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS,
+        ).match("set is not an instance of int")
+
+
+class TestFrozenSet:
+    def test_bad_type(self):
+        pytest.raises(TypeCheckError, check_type, 5, FrozenSet[int]).match(
+            "int is not a frozenset"
         )
-        pytest.raises(TypeCheckError, check_type, {1, 2, "bb"}, Set[int]).match(
-            "set is not an instance of int"
+
+    def test_valid(self):
+        check_type(frozenset({1, 2}), FrozenSet[int])
+
+    def test_first_check_empty(self):
+        check_type(frozenset(), FrozenSet[int])
+
+    def test_first_check_fail(self, sample_set: set):
+        pytest.raises(
+            TypeCheckError, check_type, frozenset(sample_set), FrozenSet[int]
+        ).match("set is not an instance of int")
+
+    def test_full_check_fail(self):
+        pytest.raises(
+            TypeCheckError,
+            check_type,
+            frozenset({1, 2, "bb"}),
+            FrozenSet[int],
+            collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS,
+        ).match("set is not an instance of int")
+
+    def test_set_against_frozenset(self, sample_set: set):
+        pytest.raises(TypeCheckError, check_type, sample_set, FrozenSet[int]).match(
+            "set is not a frozenset"
         )
 
 
@@ -646,17 +747,13 @@ class TestTuple:
             TypeCheckError, check_type, ("blah",), annotated_type[int, ...]
         ).match("tuple is not an instance of int")
 
-    def test_ellipsis_bad_element_full_check(
-        self, annotated_type: Any, monkeypatch: MonkeyPatch
-    ):
-        monkeypatch.setattr(
-            config, "collection_check_strategy", CollectionCheckStrategy.ALL_ITEMS
-        )
+    def test_ellipsis_bad_element_full_check(self, annotated_type: Any):
         pytest.raises(
             TypeCheckError,
             check_type,
             (1, 2, "blah"),
             annotated_type[int, ...],
+            collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS,
         ).match("tuple is not an instance of int")
 
     def test_empty_tuple(self, annotated_type: Any):
@@ -723,6 +820,93 @@ class TestUnion:
             f"  int: is not an instance of int"
         )
 
+    @pytest.mark.skipif(
+        sys.implementation.name != "cpython",
+        reason="Test relies on CPython's reference counting behavior",
+    )
+    def test_union_reference_leak(self):
+        class Leak:
+            def __del__(self):
+                nonlocal leaked
+                leaked = False
+
+        def inner1():
+            leak = Leak()  # noqa: F841
+            check_type(b"asdf", Union[str, bytes])
+
+        leaked = True
+        inner1()
+        assert not leaked
+
+        def inner2():
+            leak = Leak()  # noqa: F841
+            check_type(b"asdf", Union[bytes, str])
+
+        leaked = True
+        inner2()
+        assert not leaked
+
+        def inner3():
+            leak = Leak()  # noqa: F841
+            with pytest.raises(TypeCheckError, match="any element in the union:"):
+                check_type(1, Union[str, bytes])
+
+        leaked = True
+        inner3()
+        assert not leaked
+
+    @pytest.mark.skipif(
+        sys.implementation.name != "cpython",
+        reason="Test relies on CPython's reference counting behavior",
+    )
+    @pytest.mark.skipif(sys.version_info < (3, 10), reason="UnionType requires 3.10")
+    def test_uniontype_reference_leak(self):
+        class Leak:
+            def __del__(self):
+                nonlocal leaked
+                leaked = False
+
+        def inner1():
+            leak = Leak()  # noqa: F841
+            check_type(b"asdf", str | bytes)
+
+        leaked = True
+        inner1()
+        assert not leaked
+
+        def inner2():
+            leak = Leak()  # noqa: F841
+            check_type(b"asdf", bytes | str)
+
+        leaked = True
+        inner2()
+        assert not leaked
+
+        def inner3():
+            leak = Leak()  # noqa: F841
+            with pytest.raises(TypeCheckError, match="any element in the union:"):
+                check_type(1, Union[str, bytes])
+
+        leaked = True
+        inner3()
+        assert not leaked
+
+    @pytest.mark.skipif(sys.version_info < (3, 10), reason="UnionType requires 3.10")
+    def test_raw_uniontype_success(self):
+        check_type(str | int, types.UnionType)
+
+    @pytest.mark.skipif(sys.version_info < (3, 10), reason="UnionType requires 3.10")
+    def test_raw_uniontype_fail(self):
+        if sys.version_info < (3, 14):
+            expected_type = r"\w+\.UnionType"
+        else:
+            expected_type = "Union"
+
+        with pytest.raises(
+            TypeCheckError, match=f"class str is not an instance of {expected_type}$"
+        ):
+            check_type(str, types.UnionType)
+
 
 class TestTypevar:
     def test_bound(self):
@@ -751,14 +935,22 @@ class TestTypevar:
 
 
 class TestNewType:
-    def test_valid(self):
+    def test_simple_valid(self):
         check_type(1, myint)
 
-    def test_bad_value(self):
+    def test_simple_bad_value(self):
         pytest.raises(TypeCheckError, check_type, "a", myint).match(
             r"str is not an instance of int"
         )
 
+    def test_generic_valid(self):
+        check_type([1], mylist)
+
+    def test_generic_bad_value(self):
+        pytest.raises(TypeCheckError, check_type, ["a"], mylist).match(
+            r"item 0 of list is not an instance of int"
+        )
+
 
 class TestType:
     @pytest.mark.parametrize("annotation", [pytest.param(Type), pytest.param(type)])
@@ -808,6 +1000,12 @@ class TestType:
         T = TypeVar("T", bound=Parent)
         check_type(Child, Type[T])
 
+    @pytest.mark.parametrize("check_against", [type, Type[Any]])
+    def test_generic_aliase(self, check_against):
+        check_type(dict[str, str], check_against)
+        check_type(Dict, check_against)
+        check_type(Dict[str, str], check_against)
+
 
 class TestIO:
     @pytest.mark.parametrize(
@@ -857,63 +1055,352 @@ class TestIO:
             check_type(f, TextIO)
 
 
+class TestIntersectingProtocol:
+    SIT = TypeVar("SIT", covariant=True)
+
+    class SizedIterable(
+        Sized,
+        Iterable[SIT],
+        Protocol[SIT],
+    ): ...
+
+    @pytest.mark.parametrize(
+        "subject, predicate_type",
+        (
+            pytest.param(
+                (),
+                SizedIterable,
+                id="empty_tuple_unspecialized",
+            ),
+            pytest.param(
+                range(2),
+                SizedIterable,
+                id="range",
+            ),
+            pytest.param(
+                (),
+                SizedIterable[int],
+                id="empty_tuple_int_specialized",
+            ),
+            pytest.param(
+                (1, 2, 3),
+                SizedIterable[int],
+                id="tuple_int_specialized",
+            ),
+            pytest.param(
+                ("1", "2", "3"),
+                SizedIterable[str],
+                id="tuple_str_specialized",
+            ),
+        ),
+    )
+    def test_valid_member_passes(self, subject: object, predicate_type: type) -> None:
+        for _ in range(2):  # Makes sure that the cache is also exercised
+            check_type(subject, predicate_type)
+
+    xfail_nested_protocol_checks = pytest.mark.xfail(
+        reason="false negative due to missing support for nested protocol checks",
+    )
+
+    @pytest.mark.parametrize(
+        "subject, predicate_type",
+        (
+            pytest.param(
+                (1 for _ in ()),
+                SizedIterable,
+                id="generator",
+            ),
+            pytest.param(
+                range(2),
+                SizedIterable[str],
+                marks=xfail_nested_protocol_checks,
+                id="range_str_specialized",
+            ),
+            pytest.param(
+                (1, 2, 3),
+                SizedIterable[str],
+                marks=xfail_nested_protocol_checks,
+                id="int_tuple_str_specialized",
+            ),
+            pytest.param(
+                ("1", "2", "3"),
+                SizedIterable[int],
+                marks=xfail_nested_protocol_checks,
+                id="str_tuple_int_specialized",
+            ),
+        ),
+    )
+    def test_raises_for_non_member(self, subject: object, predicate_type: type) -> None:
+        with pytest.raises(TypeCheckError):
+            check_type(subject, predicate_type)
+
+
 class TestProtocol:
-    def test_protocol(self):
+    @pytest.mark.parametrize(
+        "instantiate",
+        [pytest.param(True, id="instance"), pytest.param(False, id="class")],
+    )
+    def test_success(self, typing_provider: Any, instantiate: bool) -> None:
+        class MyProtocol(Protocol):
+            member: int
+
+            def noargs(self) -> None:
+                pass
+
+            def posonlyargs(self, a: int, b: str, /) -> None:
+                pass
+
+            def posargs(self, a: int, b: str, c: float = 2.0) -> None:
+                pass
+
+            def varargs(self, *args: Any) -> None:
+                pass
+
+            def varkwargs(self, **kwargs: Any) -> None:
+                pass
+
+            def varbothargs(self, *args: Any, **kwargs: Any) -> None:
+                pass
+
+            @staticmethod
+            def my_static_method(x: int, y: str) -> None:
+                pass
+
+            @classmethod
+            def my_class_method(cls, x: int, y: str) -> None:
+                pass
+
         class Foo:
             member = 1
 
+            def noargs(self, x: int = 1) -> None:
+                pass
+
+            def posonlyargs(self, a: int, b: str, c: float = 2.0, /) -> None:
+                pass
+
+            def posargs(self, *args: Any) -> None:
+                pass
+
+            def varargs(self, *args: Any, kwarg: str = "foo") -> None:
+                pass
+
+            def varkwargs(self, **kwargs: Any) -> None:
+                pass
+
+            def varbothargs(self, *args: Any, **kwargs: Any) -> None:
+                pass
+
+            # These were intentionally reversed, as this is OK for mypy
+            @classmethod
+            def my_static_method(cls, x: int, y: str) -> None:
+                pass
+
+            @staticmethod
+            def my_class_method(x: int, y: str) -> None:
+                pass
+
+        if instantiate:
+            check_type(Foo(), MyProtocol)
+        else:
+            check_type(Foo, type[MyProtocol])
+
+    @pytest.mark.parametrize(
+        "instantiate",
+        [pytest.param(True, id="instance"), pytest.param(False, id="class")],
+    )
+    @pytest.mark.parametrize("subject_class", [object, str, Parent])
+    def test_empty_protocol(self, instantiate: bool, subject_class: type[Any]):
+        class EmptyProtocol(Protocol):
+            pass
+
+        if instantiate:
+            check_type(subject_class(), EmptyProtocol)
+        else:
+            check_type(subject_class, type[EmptyProtocol])
+
+    @pytest.mark.parametrize("has_member", [True, False])
+    def test_member_checks(self, has_member: bool) -> None:
+        class MyProtocol(Protocol):
+            member: int
+
+        class Foo:
+            def __init__(self, member: int):
+                if member:
+                    self.member = member
+
+        if has_member:
+            check_type(Foo(1), MyProtocol)
+        else:
+            pytest.raises(TypeCheckError, check_type, Foo(0), MyProtocol).match(
+                f"^{qualified_name(Foo)} is not compatible with the "
+                f"{MyProtocol.__qualname__} protocol because it has no attribute named "
+                f"'member'"
+            )
+
+    def test_missing_method(self) -> None:
+        class MyProtocol(Protocol):
             def meth(self) -> None:
                 pass
 
-        check_type(Foo(), RuntimeProtocol)
-        check_type(Foo, Type[RuntimeProtocol])
-
-    def test_protocol_warns_on_static(self):
         class Foo:
-            member = 1
+            pass
 
+        pytest.raises(TypeCheckError, check_type, Foo(), MyProtocol).match(
+            f"^{qualified_name(Foo)} is not compatible with the "
+            f"{MyProtocol.__qualname__} protocol because it has no method named "
+            f"'meth'"
+        )
+
+    def test_too_many_posargs(self) -> None:
+        class MyProtocol(Protocol):
             def meth(self) -> None:
                 pass
 
-        with pytest.warns(
-            UserWarning, match=r"Typeguard cannot check the StaticProtocol protocol.*"
-        ):
-            check_type(Foo(), StaticProtocol)
-
-        with pytest.warns(
-            UserWarning, match=r"Typeguard cannot check the StaticProtocol protocol.*"
-        ):
-            check_type(Foo, Type[StaticProtocol])
-
-    def test_fail_non_method_members(self):
         class Foo:
-            val = 1
+            def meth(self, x: str) -> None:
+                pass
 
+        pytest.raises(TypeCheckError, check_type, Foo(), MyProtocol).match(
+            f"^{qualified_name(Foo)} is not compatible with the "
+            f"{MyProtocol.__qualname__} protocol because its 'meth' method has too "
+            f"many mandatory positional arguments"
+        )
+
+    def test_wrong_posarg_name(self) -> None:
+        class MyProtocol(Protocol):
+            def meth(self, x: str) -> None:
+                pass
+
+        class Foo:
+            def meth(self, y: str) -> None:
+                pass
+
+        pytest.raises(TypeCheckError, check_type, Foo(), MyProtocol).match(
+            rf"^{qualified_name(Foo)} is not compatible with the "
+            rf"{MyProtocol.__qualname__} protocol because its 'meth' method has a "
+            rf"positional argument \(y\) that should be named 'x' at this position"
+        )
+
+    def test_too_few_posargs(self) -> None:
+        class MyProtocol(Protocol):
+            def meth(self, x: str) -> None:
+                pass
+
+        class Foo:
             def meth(self) -> None:
                 pass
 
-        clsname = f"{__name__}.TestProtocol.test_fail_non_method_members.<locals>.Foo"
-        pytest.raises(TypeCheckError, check_type, Foo(), RuntimeProtocol).match(
-            f"{clsname} is not compatible with the RuntimeProtocol protocol"
-        )
-        pytest.raises(TypeCheckError, check_type, Foo, Type[RuntimeProtocol]).match(
-            f"class {clsname} is not compatible with the RuntimeProtocol protocol"
+        pytest.raises(TypeCheckError, check_type, Foo(), MyProtocol).match(
+            f"^{qualified_name(Foo)} is not compatible with the "
+            f"{MyProtocol.__qualname__} protocol because its 'meth' method has too "
+            f"few positional arguments"
         )
 
-    def test_fail(self):
-        class Foo:
-            def meth2(self) -> None:
+    def test_no_varargs(self) -> None:
+        class MyProtocol(Protocol):
+            def meth(self, *args: Any) -> None:
                 pass
 
-        pattern = (
-            f"{__name__}.TestProtocol.test_fail.<locals>.Foo is not compatible with "
-            f"the RuntimeProtocol protocol"
+        class Foo:
+            def meth(self) -> None:
+                pass
+
+        pytest.raises(TypeCheckError, check_type, Foo(), MyProtocol).match(
+            f"^{qualified_name(Foo)} is not compatible with the "
+            f"{MyProtocol.__qualname__} protocol because its 'meth' method should "
+            f"accept variable positional arguments but doesn't"
         )
-        pytest.raises(TypeCheckError, check_type, Foo(), RuntimeProtocol).match(pattern)
-        pytest.raises(TypeCheckError, check_type, Foo, Type[RuntimeProtocol]).match(
-            pattern
+
+    def test_no_kwargs(self) -> None:
+        class MyProtocol(Protocol):
+            def meth(self, **kwargs: Any) -> None:
+                pass
+
+        class Foo:
+            def meth(self) -> None:
+                pass
+
+        pytest.raises(TypeCheckError, check_type, Foo(), MyProtocol).match(
+            f"^{qualified_name(Foo)} is not compatible with the "
+            f"{MyProtocol.__qualname__} protocol because its 'meth' method should "
+            f"accept variable keyword arguments but doesn't"
         )
 
+    def test_missing_kwarg(self) -> None:
+        class MyProtocol(Protocol):
+            def meth(self, *, x: str) -> None:
+                pass
+
+        class Foo:
+            def meth(self) -> None:
+                pass
+
+        pytest.raises(TypeCheckError, check_type, Foo(), MyProtocol).match(
+            f"^{qualified_name(Foo)} is not compatible with the "
+            f"{MyProtocol.__qualname__} protocol because its 'meth' method is "
+            f"missing keyword-only arguments: x"
+        )
+
+    def test_extra_kwarg(self) -> None:
+        class MyProtocol(Protocol):
+            def meth(self) -> None:
+                pass
+
+        class Foo:
+            def meth(self, *, x: str) -> None:
+                pass
+
+        pytest.raises(TypeCheckError, check_type, Foo(), MyProtocol).match(
+            f"^{qualified_name(Foo)} is not compatible with the "
+            f"{MyProtocol.__qualname__} protocol because its 'meth' method has "
+            f"mandatory keyword-only arguments not present in the protocol: x"
+        )
+
+    def test_instance_staticmethod_mismatch(self) -> None:
+        class MyProtocol(Protocol):
+            @staticmethod
+            def meth() -> None:
+                pass
+
+        class Foo:
+            def meth(self) -> None:
+                pass
+
+        pytest.raises(TypeCheckError, check_type, Foo(), MyProtocol).match(
+            f"^{qualified_name(Foo)} is not compatible with the "
+            f"{MyProtocol.__qualname__} protocol because its 'meth' method should "
+            f"be a static method but it's an instance method"
+        )
+
+    def test_instance_classmethod_mismatch(self) -> None:
+        class MyProtocol(Protocol):
+            @classmethod
+            def meth(cls) -> None:
+                pass
+
+        class Foo:
+            def meth(self) -> None:
+                pass
+
+        pytest.raises(TypeCheckError, check_type, Foo(), MyProtocol).match(
+            f"^{qualified_name(Foo)} is not compatible with the "
+            f"{MyProtocol.__qualname__} protocol because its 'meth' method should "
+            f"be a class method but it's an instance method"
+        )
+
+    def test_builtin_signature_check(self) -> None:
+        class MyProtocol(Protocol):
+            def attr(self) -> None:
+                pass
+
+        class Foo:
+            attr = timedelta
+
+        # Foo.attr is incompatible but timedelta has not inspectable signature so the
+        # check is skipped
+        check_type(Foo(), MyProtocol)
+
 
 class TestRecursiveType:
     def test_valid(self):
@@ -981,15 +1468,27 @@ class TestTypeGuard:
     ],
 )
 def test_forward_reference_policy(
-    policy: ForwardRefPolicy, contextmanager: ContextManager, monkeypatch: MonkeyPatch
+    policy: ForwardRefPolicy, contextmanager: ContextManager
 ):
-    monkeypatch.setattr(config, "forward_ref_policy", policy)
     with contextmanager:
-        check_type(1, ForwardRef("Foo"))  # noqa: F821
+        check_type(1, ForwardRef("Foo"), forward_ref_policy=policy)  # noqa: F821
 
 
 def test_any():
-    check_type("aa", Any)
+    assert check_type("aa", Any) == "aa"
+
+
+def test_suppressed_checking():
+    with suppress_type_checks():
+        assert check_type("aa", int) == "aa"
+
+
+def test_suppressed_checking_exception():
+    with pytest.raises(RuntimeError), suppress_type_checks():
+        assert check_type("aa", int) == "aa"
+        raise RuntimeError
+
+    pytest.raises(TypeCheckError, check_type, "aa", int)
 
 
 def test_any_subclass():
@@ -999,6 +1498,26 @@ def test_any_subclass():
     check_type(Foo(), int)
 
 
+def test_none():
+    check_type(None, None)
+
+
 def test_return_checked_value():
     value = {"foo": 1}
     assert check_type(value, Dict[str, int]) is value
+
+
+def test_imported_str_forward_ref():
+    value = {"foo": 1}
+    memo = TypeCheckMemo(globals(), locals())
+    pattern = r"Skipping type check against 'Dict\[str, int\]'"
+    with pytest.warns(TypeHintWarning, match=pattern):
+        check_type_internal(value, "Dict[str, int]", memo)
+
+
+def test_check_against_tuple_success():
+    check_type(1, (float, Union[str, int]))
+
+
+def test_check_against_tuple_failure():
+    pytest.raises(TypeCheckError, check_type, "aa", (int, bytes))
